from microGUI.Widgets.Widget import Widget


class Basic(Widget):
    """
    A superclass of basic resources for most widgets.

    The :class:`~microGUI.Widgets.Basic.Basic` superclass provides basic resources for all widgets.

    It provides the fundamental events for:
     - getting/losing focus activating button press
     - release
     - repeat

    Also, Basic supports:
     - toggle buttons
     - autohighlighting

    and provides properties for:
     - margins
     - bevel colors
     - outline and inline colors
     - draw color
     - fill color
     - fill pattern.

    """
    def __init__(self):
        super().__init__()



from microGUI.Widgets.Basic import Basic


class Label(Basic):
    """
    A text, bitmap, or image label.

    The Label class provides a text string, bitmap, or image for labeling other widgets. You can have text pop up in
    a balloon to provide further meaning to the label.

    .. seealso::
     - :py:class:`~microGUI.Widgets.Button.Button`
     - :py:attr:`~microGUI.Widgets.Button.Button.Text`
    """
    def __init__(self):
        Basic.__init__(self)





from microGUI.mainloop import MainLoop


# 1
from microGUI.Widgets import Widget

# 2
from microGUI.Widgets.Basic import Basic
from microGUI.Widgets.Timer import Timer

# 3
from microGUI.Widgets.bargraph import BarGraph
from microGUI.Widgets.calendar import Calendar
from microGUI.Widgets.clock import Clock
from microGUI.Widgets.Container import Container
from microGUI.Widgets.gauge import Gauge
from microGUI.Widgets.graphic import Graphic
from microGUI.Widgets.Label import Label
from microGUI.Widgets.meter import Meter
from microGUI.Widgets.mtrend import MTrend
from microGUI.Widgets.raw import Raw
from microGUI.Widgets.separator import Separator
from microGUI.Widgets.trend import Trend
from microGUI.Widgets.updown import UpDown

# 4
from microGUI.Widgets.bkgd import Bkgd
from microGUI.Widgets.client import Client
from microGUI.Widgets.compound import Compound
from microGUI.Widgets.disjoint import Disjoint
from microGUI.Widgets.fontsel import FontSel
from microGUI.Widgets.group import Group
from microGUI.Widgets.imagearea import ImageArea
from microGUI.Widgets.oscontainer import OsContainer
from microGUI.Widgets.pane import Pane
from microGUI.Widgets.panelgroup import PanelGroup
from microGUI.Widgets.printsel import PrintSel
from microGUI.Widgets.region import Region
from microGUI.Widgets.scrollarea import ScrollArea

APPLICATION_VERSION = "0.1a1"
APPLICATION_AUTHORS = "Tuuux, Mo"

mainloop = MainLoop()


__all__ = [
    "mainloop"
]





from microGUI.Widgets import AnchorStyles


class Widget:
    """
    Widget is the fundamental superclass. All widgets belong to a subclass of Widget.

    .. seealso:: dynamic_widget_caste()
    """

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.__AllowDrop = None
        self.AllowDrop = None

        self.__Anchor = None
        self.Anchor = None

        self.__BevelWidth = None
        self.BevelWidth = None

    @property
    def AllowDrop(self):
        """
        Gets or sets a value indicating whether the widget can accept data that the user drags onto it.

        .. seealso::
         - DragEventArgs
         - DragOver
         - DragDrop
         - DragEnter
         - DragLeave
         - DoDragDrop

        :getter: Return ``true`` if drag-and-drop operations are allowed in the widget; otherwise, ``false``.
                 The default is ``false``.
        :setter: Sets AllowDrop property
        :raise TypeError: When not set AllowDrop with a bool type or None
        :type: bool
        """
        return self.__AllowDrop

    @AllowDrop.setter
    def AllowDrop(self, value):
        if value is None:
            value = False
        if value is not None and not isinstance(value, bool):
            raise TypeError("'AllowDrop' property value must be a bool type or None")
        if self.AllowDrop != value:
            self.__AllowDrop = value

    @property
    def Anchor(self):
        """
        Gets or sets a value indicating whether the widget can accept data that the user drags onto it.

        Exemple
        =======

        .. code-block:: python
          :emphasize-lines: 2

          widget = Widget()
          widget.Anchor = ( Widgets.AnchorStyles.Bottom | Widgets.AnchorStyles.Right )

        .. attention::
         Use the Widget.Anchor property to define how a widget is automatically resized as its parent widget is \
         resized. Anchoring a widget to its parent widget ensures that the anchored edges remain in the same \
         position relative to the edges of the parent widget when the parent widget is resized. \
         You can anchor a widget to one or more edges of its container. For example, if you have a Container \
         with a Button whose microGUI.Widgets.Button.Anchor property value is set to microGUI.Widgets.AnchorStyles.Top
         and :py:class:`microGUI.Widgets.AnchorStyles.Bottom`, \
         the microGUI.Widgets.Button is stretched to maintain the anchored distance to the top and bottom edges of the \
         :py:class:`~microGUI.Widgets.Container.Container` as the Height of the \
         :py:class:`~microGUI.Widgets.Container.Container` is increased.

        .. seealso::
         - :py:class:`microGUI.Widgets.AnchorStyles`
         - Dock
         - Layout

        :getter: Return ``True`` if drag-and-drop operations are allowed in the widget; otherwise, ``False``.
                 The default is ``False``.
        :setter: Sets :py:attr:`~microGUI.Widgets.Widget.Widget.Anchor` property
        :raise TypeError: When not set AllowDrop with a bool type or None
        :type: A bitwise combination of the Flags :py:class:`microGUI.Widgets.AnchorStyles` values.
        """
        return self.__Anchor

    @Anchor.setter
    def Anchor(self, value):
        if value is None:
            value = AnchorStyles.NoneAnchoredNone
        if value is not None and not isinstance(value, AnchorStyles):
            raise TypeError("'Anchor' property value must be a AnchorStyles type or None")
        if self.Anchor != value:
            self.__Anchor = value

    @property
    def BevelWidth(self):
        """
        Gets or sets the bevel width of the widget.

        .. attention::
         The width of the widget's bevel if the widget is highlighted and is to draw a bevel.

        .. seealso::
         - :py:class:`microGUI.Widgets.Widget.ThisFlagsHighlighted`
         - :py:class:`microGUI.Widgets.Basic.ThisFlags.Basic`

        :getter: The bevel width of the widget in chars.
        :setter: Sets :py:attr:`~microGUI.Widgets.Widget.Widget.BevelWidth` property
        :raise TypeError: When not set BevelWidth with a int type or None
        :type: int
        """
        return self.__BevelWidth

    @BevelWidth.setter
    def BevelWidth(self, value):
        if value is None:
            value = 0
        if value is not None and not isinstance(value, int):
            raise TypeError("'BevelWidth' property value must be a int type or None")
        if self.BevelWidth != value:
            self.__BevelWidth = value

    def get(self):
        """
        Returns the stored pointer.

        :return: self from widget point of view
        :rtype: Widget
        """
        return self

    # def Contains(self, widget) -> bool:
    #     """
    #     Retrieves a value indicating whether the specified widget is a child of the widget.
    #
    #     :Example:
    #
    #     def MakeLabelVisible():
    #         if main_window.Contains(label1):
    #             label1.BringToFront()
    #
    #
    #     .. seealso:: BringToFront() Send
    #
    #     :param widget: The Widget to evaluate
    #     :type widget: Widget
    #     :return: ``true`` if the specified widget is a child of the widget; otherwise, ``false``.
    #     :rtype: bool
    #     """
    #     raise NotImplemented




class MainLoop:

    def __init__(self):
        self.__running = None
        self.running = None

    @property
    def running(self) -> bool:
        """
        Gets or sets a value indicating whether the widget can accept data that the user drags onto it.

        :getter: Return ``true`` if the mainloop operations are running; otherwise, false. The default \
                 is ``false``.
        :setter: Sets running property
        :raise TypeError: When not set running with a bool type or None
        :type: bool
        """
        return self.__running

    @running.setter
    def running(self, value):
        if value is None:
            value = False
        if value is not None and not isinstance(value, bool):
            raise TypeError("'running' property value must be a bool type or None")
        if self.running != value:
            self.__running = value

    def start(self):
        self.running = True

    def stop(self):
        self.running = False



from enum import Flag, auto


class AnchorStyles(Flag):
    """
    Anchor flags for :class:`~microGUI.Widgets.Widget.Widget` resource :func:`microGUI.Widget.Arguments.anchor_flags`.

    Anchor flags specify how the widget is anchored to its parent.

   .. list-table:: **Enumerator**
      :widths: 25 75
      :header-rows: 0
      :class: tight-table

      * - **LeftAnchoredRight**
        - Anchors the widget's left extent to the right edge of its parent's canvas.
      * - **RightAnchoredRight**
        - Anchors the widget's right extent to the right edge of its parent's canvas.
      * - **TopAnchoredBottom**
        - Anchors the widget's top extent to the bottom edge of its parent's canvas.
      * - **BottomAnchoredBottom**
        - Anchors the widget's bottom extent to the bottom edge of its parent's canvas.
      * - **LeftAnchoredLeft**
        - Anchors the widget's left extent to the left edge of its parent's canvas.
      * - **RightAnchoredLeft**
        - Anchors the widget's right extent to the left edge of its parent's canvas.
      * - **TopAnchoredTop**
        - Anchors the widget's top extent to the top edge of its parent's canvas.
      * - **BottomAnchoredTop**
        - Anchors the widget's bottom extent to the top edge of its parent's canvas.
      * - **BalloonsOn**
        - If a child widget has been assigned a balloon, pop up the balloon as soon as the pointer passes over the
          child widget; otherwise delay the pop-up for 1.25 seconds.
      * - **Bottom**
        - The widget is anchored to the bottom edge of its container.
      * - **Left**
        - The widget is anchored to the left edge of its container.
      * - **NoWhere**
        - The widget is not anchored to any edges of its container.
      * - **Right**
        - The widget is anchored to the right edge of its container.
      * - **Top**
        - The widget is anchored to the top edge of its container.

    """
    LeftAnchoredRight = auto()
    RightAnchoredRight = auto()
    TopAnchoredBottom = auto()
    BottomAnchoredBottom = auto()
    LeftAnchoredLeft = auto()
    RightAnchoredLeft = auto()
    TopAnchoredTop = auto()
    BottomAnchoredTop = auto()
    BalloonsOn = auto()
    Bottom = BottomAnchoredBottom
    Left = LeftAnchoredLeft
    NoneAnchoredNone = auto()
    Right = RightAnchoredRight
    Top = TopAnchoredTop
    All = LeftAnchoredRight | RightAnchoredRight | TopAnchoredBottom | BottomAnchoredBottom | LeftAnchoredLeft | RightAnchoredLeft | TopAnchoredTop | BottomAnchoredTop | BalloonsOn


class BalloonPosition(Flag):
    """
    Indicates where the balloon with the :class:`~microGUI.Widgets.Label.Label`'s text pops up.

    Apply to :py:attr:`microGUI.Widgets.Label.Label.BalloonPosition` property.

   .. seealso::
    - :py:attr:`microGUI.Widgets.Label.Label.BalloonPosition`
    - :py:attr:`microGUI.Widgets.Label.Label.Arguments.balloon_position`

   .. list-table:: **Enumerator**
      :widths: 25 75
      :header-rows: 0
      :class: tight-table

      * - **Inplace**
        - keep :class:`~microGUI.Widgets.Label.Label` balloon inplace.
      * - **Top**
        - place :class:`~microGUI.Widgets.Label.Label` balloon to top.
      * - **Left**
        - place :class:`~microGUI.Widgets.Label.Label` balloon to left.
      * - **Right**
        - place :class:`~microGUI.Widgets.Label.Label` balloon to right.
      * - **Bottom**
        - place :class:`~microGUI.Widgets.Label.Label` balloon to bottom.
    """
    Inplace = auto()
    Top = auto()
    Left = auto()
    Right = auto()
    Bottom = auto()



import unittest
from microGUI.mainloop import MainLoop


class TestMainLoop(unittest.TestCase):

    def setUp(self) -> None:
        self.mainloop = MainLoop()

    def test_running(self):
        self.assertFalse(self.mainloop.running)

        self.mainloop.running = True
        self.assertTrue(self.mainloop.running)

        self.mainloop.running = False
        self.assertFalse(self.mainloop.running)

        self.assertRaises(TypeError, setattr, self.mainloop, "running", 42)

    def test_run(self):
        self.assertFalse(self.mainloop.running)

        self.mainloop.start()
        self.assertTrue(self.mainloop.running)

    def test_stop(self):
        self.assertFalse(self.mainloop.running)

        self.mainloop.running = True
        self.assertTrue(self.mainloop.running)

        self.mainloop.stop()
        self.assertFalse(self.mainloop.running)


if __name__ == '__main__':
    unittest.main()
